"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.GLOBE_VIEW_EVENTS = void 0;
var THREE = _interopRequireWildcard(require("three"));
var _View = _interopRequireWildcard(require("../View"));
var _GlobeControls = _interopRequireDefault(require("../../Controls/GlobeControls"));
var _Coordinates = _interopRequireDefault(require("../Geographic/Coordinates"));
var _GlobeLayer = _interopRequireDefault(require("./Globe/GlobeLayer"));
var _Atmosphere = _interopRequireDefault(require("./Globe/Atmosphere"));
var _CameraUtils = _interopRequireDefault(require("../../Utils/CameraUtils"));
var _Crs = _interopRequireDefault(require("../Geographic/Crs"));
var _Ellipsoid = require("../Math/Ellipsoid");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
/**
 * Fires when the view is completely loaded. Controls and view's functions can be called then.
 * @event GlobeView#initialized
 * @property target {view} dispatched on view
 * @property type {string} initialized
 */
/**
 * Fires when a layer is added
 * @event GlobeView#layer-added
 * @property layerId {string} the id of the layer
 * @property target {view} dispatched on view
 * @property type {string} layers-added
 */
/**
 * Fires when a layer is removed
 * @event GlobeView#layer-removed
 * @property layerId {string} the id of the layer
 * @property target {view} dispatched on view
 * @property type {string} layers-added
 */
/**
 * Fires when the layers oder has changed
 * @event GlobeView#layers-order-changed
 * @property new {object}
 * @property new.sequence {array}
 * @property new.sequence.0 {number} the new layer at position 0
 * @property new.sequence.1 {number} the new layer at position 1
 * @property new.sequence.2 {number} the new layer at position 2
 * @property previous {object}
 * @property previous.sequence {array}
 * @property previous.sequence.0 {number} the previous layer at position 0
 * @property previous.sequence.1 {number} the previous layer at position 1
 * @property previous.sequence.2 {number} the previous layer at position 2
 * @property target {view} dispatched on view
 * @property type {string} layers-order-changed
 */

/**
 * Globe's EVENT
 * @property GLOBE_INITIALIZED {string} Deprecated: emit one time when globe is initialized (use VIEW_EVENTS.INITIALIZED instead).
 * @property LAYER_ADDED {string} Deprecated: emit when layer id added in viewer (use VIEW_EVENTS.LAYER_ADDED instead).
 * @property LAYER_REMOVED {string} Deprecated: emit when layer id removed in viewer (use VIEW_EVENTS.LAYER_REMOVED instead).
 * @property COLOR_LAYERS_ORDER_CHANGED {string} Deprecated: emit when  color layers order change (use VIEW_EVENTS.COLOR_LAYERS_ORDER_CHANGED instead).
 */

const GLOBE_VIEW_EVENTS = {
  GLOBE_INITIALIZED: _View.VIEW_EVENTS.INITIALIZED,
  LAYER_ADDED: _View.VIEW_EVENTS.LAYER_ADDED,
  LAYER_REMOVED: _View.VIEW_EVENTS.LAYER_REMOVED,
  COLOR_LAYERS_ORDER_CHANGED: _View.VIEW_EVENTS.COLOR_LAYERS_ORDER_CHANGED
};
exports.GLOBE_VIEW_EVENTS = GLOBE_VIEW_EVENTS;
class GlobeView extends _View.default {
  /**
   * Creates a view of a globe.
   *
   * @constructor
   * @extends View
   *
   * @example <caption><b>Instance GlobeView.</b></caption>
   * var viewerDiv = document.getElementById('viewerDiv');
   * const placement = {
   *     coord: new itowns.Coordinates('EPSG:4326', 2.351323, 48.856712),
   *     range: 25000000,
   * }
   * var view = new itowns.GlobeView(viewerDiv, placement);
   *
   * @example <caption><b>Enable WebGl 1.0 instead of WebGl 2.0.</b></caption>
   * var viewerDiv = document.getElementById('viewerDiv');
   * const placement = {
   *     coord: new itowns.Coordinates('EPSG:4326', 2.351323, 48.856712),
   *     range: 25000000,
   * }
   * var view = new itowns.GlobeView(viewerDiv, placement, {  renderer: { isWebGL2: false } });
   *
   * @param {HTMLDivElement} viewerDiv - Where to attach the view and display it
   * in the DOM.
   * @param {CameraTransformOptions|Extent} placement - An object to place view
   * @param {object=} options - See options of {@link View}.
   * @param {Object} options.controls - See options of {@link GlobeControls}
   */
  constructor(viewerDiv) {
    let placement = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    let options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    THREE.Object3D.DEFAULT_UP.set(0, 0, 1);
    // Setup View
    super('EPSG:4978', viewerDiv, options);
    this.isGlobeView = true;
    this.camera3D.near = Math.max(15.0, 0.000002352 * _Ellipsoid.ellipsoidSizes.x);
    this.camera3D.far = _Ellipsoid.ellipsoidSizes.x * 10;
    const tileLayer = new _GlobeLayer.default('globe', options.object3d, options);
    this.mainLoop.gfxEngine.label2dRenderer.infoTileLayer = tileLayer.info;
    const sun = new THREE.DirectionalLight();
    sun.position.set(-0.5, 0, 1);
    sun.updateMatrixWorld(true);
    this.scene.add(sun);
    this.addLayer(tileLayer);
    this.tileLayer = tileLayer;
    if (!placement.isExtent) {
      placement.coord = placement.coord || new _Coordinates.default('EPSG:4326', 0, 0);
      placement.tilt = placement.tilt || 89.5;
      placement.heading = placement.heading || 0;
      placement.range = placement.range || _Ellipsoid.ellipsoidSizes.x * 2.0;
    }
    if (options.noControls) {
      _CameraUtils.default.transformCameraToLookAtTarget(this, this.camera3D, placement);
    } else {
      this.controls = new _GlobeControls.default(this, placement, options.controls);
      this.controls.handleCollision = typeof options.handleCollision !== 'undefined' ? options.handleCollision : true;
    }
    this.addLayer(new _Atmosphere.default('atmosphere', options.atmosphere));

    // GlobeView needs this.camera.resize to set perpsective matrix camera
    this.camera.resize(viewerDiv.clientWidth, viewerDiv.clientHeight);
  }

  /**
   * Add layer in viewer.
   * The layer id must be unique.
   *
   * The `layer.whenReady` is a promise that resolves when
   * the layer is done. This promise is also returned by
   * `addLayer` allowing to chain call.
   *
   * The layer added is attached, by default to `GlobeLayer` (`GlobeView.tileLayer`).
   * If you want add a unattached layer use `View#addLayer` parent method.
   *
   * @param {LayerOptions|Layer|GeometryLayer} layer The layer to add in view.
   * @return {Promise} a promise resolved with the new layer object when it is fully initialized or rejected if any error occurred.
   */
  addLayer(layer) {
    if (!layer || !layer.isLayer) {
      return Promise.reject(new Error('Add Layer type object'));
    }
    if (layer.isColorLayer) {
      if (!this.tileLayer.tileMatrixSets.includes(_Crs.default.formatToTms(layer.source.crs))) {
        return layer._reject(`Only ${this.tileLayer.tileMatrixSets} tileMatrixSet are currently supported for color layers`);
      }
    } else if (layer.isElevationLayer) {
      if (_Crs.default.formatToTms(layer.source.crs) !== this.tileLayer.tileMatrixSets[0]) {
        return layer._reject(`Only ${this.tileLayer.tileMatrixSets[0]} tileMatrixSet is currently supported for elevation layers`);
      }
    }
    return super.addLayer(layer, this.tileLayer);
  }
  getPixelsToDegrees() {
    let pixels = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
    let screenCoord = arguments.length > 1 ? arguments[1] : undefined;
    return this.getMetersToDegrees(this.getPixelsToMeters(pixels, screenCoord));
  }
  getPixelsToDegreesFromDistance() {
    let pixels = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
    let distance = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
    return this.getMetersToDegrees(this.getPixelsToMetersFromDistance(pixels, distance));
  }
  getMetersToDegrees() {
    let meters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
    return THREE.MathUtils.radToDeg(2 * Math.asin(meters / (2 * _Ellipsoid.ellipsoidSizes.x)));
  }
}
var _default = GlobeView;
exports.default = _default;